@@ -37,7 +37,13 @@
 #include "config.h"
 
 #define KEY_ROOT HKEY_LOCAL_MACHINE
-#define KEY_PREFIX "SOFTWARE\\wpa_supplicant"
+#define KEY_PREFIX TEXT("SOFTWARE\\wpa_supplicant")
+
+#ifdef UNICODE
+#define TSTR "%S"
+#else /* UNICODE */
+#define TSTR "%s"
+#endif /* UNICODE */
 
 
 static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
@@ -48,7 +54,7 @@ static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
 	LONG ret;
 	DWORD i;
 
-	ret = RegOpenKeyEx(hk, "blobs", 0, KEY_QUERY_VALUE, &bhk);
+	ret = RegOpenKeyEx(hk, TEXT("blobs"), 0, KEY_QUERY_VALUE, &bhk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_DEBUG, "Could not open wpa_supplicant config "
 			   "blobs key");
@@ -56,13 +62,15 @@ static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
 	}
 
 	for (i = 0; ; i++) {
-		char name[255], data[4096];
+#define TNAMELEN 255
+		TCHAR name[TNAMELEN];
+		char data[4096];
 		DWORD namelen, datalen, type;
 
-		namelen = sizeof(name);
+		namelen = TNAMELEN;
 		datalen = sizeof(data);
 		ret = RegEnumValue(bhk, i, name, &namelen, NULL, &type,
-				   data, &datalen);
+				   (LPBYTE) data, &datalen);
 
 		if (ret == ERROR_NO_MORE_ITEMS)
 			break;
@@ -73,13 +81,13 @@ static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
 			break;
 		}
 
-		if (namelen >= sizeof(name))
-			namelen = sizeof(name) - 1;
-		name[namelen] = '\0';
+		if (namelen >= TNAMELEN)
+			namelen = TNAMELEN - 1;
+		name[namelen] = TEXT('\0');
+		wpa_unicode2ascii_inplace(name);
 
 		if (datalen >= sizeof(data))
 			datalen = sizeof(data) - 1;
-		data[datalen] = '\0';
 
 		wpa_printf(MSG_MSGDUMP, "blob %d: field='%s' len %d",
 			   (int) i, name, (int) datalen);
@@ -89,7 +97,7 @@ static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
 			errors++;
 			break;
 		}
-		blob->name = strdup(name);
+		blob->name = strdup((char *) name);
 		blob->data = malloc(datalen);
 		if (blob->name == NULL || blob->data == NULL) {
 			wpa_config_free_blob(blob);
@@ -108,7 +116,7 @@ static int wpa_config_read_blobs(struct wpa_config *config, HKEY hk)
 }
 
 
-static int wpa_config_read_reg_dword(HKEY hk, const char *name, int *_val)
+static int wpa_config_read_reg_dword(HKEY hk, const TCHAR *name, int *_val)
 {
 	DWORD val, buflen;
 	LONG ret;
@@ -125,20 +133,19 @@ static int wpa_config_read_reg_dword(HKEY hk, const char *name, int *_val)
 }
 
 
-static char * wpa_config_read_reg_string(HKEY hk, const char *name)
+static char * wpa_config_read_reg_string(HKEY hk, const TCHAR *name)
 {
 	DWORD buflen;
 	LONG ret;
-	char *val;
+	TCHAR *val;
 
 	buflen = 0;
 	ret = RegQueryValueEx(hk, name, NULL, NULL, NULL, &buflen);
 	if (ret != ERROR_SUCCESS)
 		return NULL;
-	val = malloc(buflen + 1);
+	val = malloc(buflen);
 	if (val == NULL)
 		return NULL;
-	val[buflen] = '\0';
 
 	ret = RegQueryValueEx(hk, name, NULL, NULL, (LPBYTE) val, &buflen);
 	if (ret != ERROR_SUCCESS) {
@@ -146,8 +153,9 @@ static char * wpa_config_read_reg_string(HKEY hk, const char *name)
 		return NULL;
 	}
 
-	wpa_printf(MSG_DEBUG, "%s=%s", name, val);
-	return val;
+	wpa_unicode2ascii_inplace(val);
+	wpa_printf(MSG_DEBUG, "%s=%s", name, (char *) val);
+	return (char *) val;
 }
 
 
@@ -155,17 +163,20 @@ static int wpa_config_read_global(struct wpa_config *config, HKEY hk)
 {
 	int errors = 0;
 
-	wpa_config_read_reg_dword(hk, "ap_scan", &config->ap_scan);
-	wpa_config_read_reg_dword(hk, "fast_reauth", &config->fast_reauth);
-	wpa_config_read_reg_dword(hk, "dot11RSNAConfigPMKLifetime",
+	wpa_config_read_reg_dword(hk, TEXT("ap_scan"), &config->ap_scan);
+	wpa_config_read_reg_dword(hk, TEXT("fast_reauth"),
+				  &config->fast_reauth);
+	wpa_config_read_reg_dword(hk, TEXT("dot11RSNAConfigPMKLifetime"),
 				  &config->dot11RSNAConfigPMKLifetime);
-	wpa_config_read_reg_dword(hk, "dot11RSNAConfigPMKReauthThreshold",
+	wpa_config_read_reg_dword(hk,
+				  TEXT("dot11RSNAConfigPMKReauthThreshold"),
 				  &config->dot11RSNAConfigPMKReauthThreshold);
-	wpa_config_read_reg_dword(hk, "dot11RSNAConfigSATimeout",
+	wpa_config_read_reg_dword(hk, TEXT("dot11RSNAConfigSATimeout"),
 				  &config->dot11RSNAConfigSATimeout);
-	wpa_config_read_reg_dword(hk, "update_config", &config->update_config);
+	wpa_config_read_reg_dword(hk, TEXT("update_config"),
+				  &config->update_config);
 
-	if (wpa_config_read_reg_dword(hk, "eapol_version",
+	if (wpa_config_read_reg_dword(hk, TEXT("eapol_version"),
 				      &config->eapol_version) == 0) {
 		if (config->eapol_version < 1 ||
 		    config->eapol_version > 2) {
@@ -175,14 +186,14 @@ static int wpa_config_read_global(struct wpa_config *config, HKEY hk)
 		}
 	}
 
-	config->ctrl_interface = wpa_config_read_reg_string(hk,
-							    "ctrl_interface");
+	config->ctrl_interface = wpa_config_read_reg_string(
+		hk, TEXT("ctrl_interface"));
 
 	return errors ? -1 : 0;
 }
 
 
-static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
+static struct wpa_ssid * wpa_config_read_network(HKEY hk, const TCHAR *netw,
 						 int id)
 {
 	HKEY nhk;
@@ -194,11 +205,11 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 	ret = RegOpenKeyEx(hk, netw, 0, KEY_QUERY_VALUE, &nhk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_DEBUG, "Could not open wpa_supplicant config "
-			   "network '%s'", netw);
+			   "network '" TSTR "'", netw);
 		return NULL;
 	}
 
-	wpa_printf(MSG_MSGDUMP, "Start of a new network '%s'", netw);
+	wpa_printf(MSG_MSGDUMP, "Start of a new network '" TSTR "'", netw);
 	ssid = wpa_zalloc(sizeof(*ssid));
 	if (ssid == NULL) {
 		RegCloseKey(nhk);
@@ -213,9 +224,9 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 		DWORD namelen, datalen, type;
 
 		namelen = 255;
-		datalen = 1024;
+		datalen = sizeof(data);
 		ret = RegEnumValue(nhk, i, name, &namelen, NULL, &type,
-				   data, &datalen);
+				   (LPBYTE) data, &datalen);
 
 		if (ret == ERROR_NO_MORE_ITEMS)
 			break;
@@ -228,13 +239,15 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 
 		if (namelen >= 255)
 			namelen = 255 - 1;
-		name[namelen] = '\0';
+		name[namelen] = TEXT('\0');
 
 		if (datalen >= 1024)
 			datalen = 1024 - 1;
-		data[datalen] = '\0';
+		data[datalen] = TEXT('\0');
 
-		if (wpa_config_set(ssid, name, data, 0) < 0)
+		wpa_unicode2ascii_inplace(name);
+		wpa_unicode2ascii_inplace(data);
+		if (wpa_config_set(ssid, (char *) name, (char *) data, 0) < 0)
 			errors++;
 	}
 
@@ -243,7 +256,7 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 	if (ssid->passphrase) {
 		if (ssid->psk_set) {
 			wpa_printf(MSG_ERROR, "Both PSK and passphrase "
-				   "configured for network '%s'.", netw);
+				   "configured for network '" TSTR "'.", netw);
 			errors++;
 		}
 		wpa_config_update_psk(ssid);
@@ -251,7 +264,8 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 
 	if ((ssid->key_mgmt & WPA_KEY_MGMT_PSK) && !ssid->psk_set) {
 		wpa_printf(MSG_ERROR, "WPA-PSK accepted for key management, "
-			   "but no PSK configured for network '%s'.", netw);
+			   "but no PSK configured for network '" TSTR "'.",
+			   netw);
 		errors++;
 	}
 
@@ -261,7 +275,7 @@ static struct wpa_ssid * wpa_config_read_network(HKEY hk, const char *netw,
 		/* Group cipher cannot be stronger than the pairwise cipher. */
 		wpa_printf(MSG_DEBUG, "Removed CCMP from group cipher "
 			   "list since it was not allowed for pairwise "
-			   "cipher for network '%s'.", netw);
+			   "cipher for network '" TSTR "'.", netw);
 		ssid->group_cipher &= ~WPA_CIPHER_CCMP;
 	}
 
@@ -282,7 +296,8 @@ static int wpa_config_read_networks(struct wpa_config *config, HKEY hk)
 	LONG ret;
 	DWORD i;
 
-	ret = RegOpenKeyEx(hk, "networks", 0, KEY_ENUMERATE_SUB_KEYS, &nhk);
+	ret = RegOpenKeyEx(hk, TEXT("networks"), 0, KEY_ENUMERATE_SUB_KEYS,
+			   &nhk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_ERROR, "Could not open wpa_supplicant networks "
 			   "registry key");
@@ -341,7 +356,7 @@ static int wpa_config_read_networks(struct wpa_config *config, HKEY hk)
 
 struct wpa_config * wpa_config_read(const char *name)
 {
-	char buf[256];
+	TCHAR buf[256];
 	int errors = 0;
 	struct wpa_config *config;
 	struct wpa_ssid *ssid;
@@ -354,7 +369,12 @@ struct wpa_config * wpa_config_read(const char *name)
 		return NULL;
 	wpa_printf(MSG_DEBUG, "Reading configuration profile '%s'", name);
 
-	snprintf(buf, sizeof(buf), KEY_PREFIX "\\configs\\%s", name);
+#ifdef UNICODE
+	_snwprintf(buf, 256, KEY_PREFIX TEXT("\\configs\\%S"), name);
+#else /* UNICODE */
+	snprintf(buf, 256, KEY_PREFIX TEXT("\\configs\\%s"), name);
+#endif /* UNICODE */
+
 	ret = RegOpenKeyEx(KEY_ROOT, buf, 0, KEY_QUERY_VALUE, &hk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_ERROR, "Could not open wpa_supplicant "
@@ -395,7 +415,7 @@ struct wpa_config * wpa_config_read(const char *name)
 }
 
 
-static int wpa_config_write_reg_dword(HKEY hk, const char *name, int val,
+static int wpa_config_write_reg_dword(HKEY hk, const TCHAR *name, int val,
 				      int def)
 {
 	LONG ret;
@@ -422,19 +442,34 @@ static int wpa_config_write_reg_string(HKEY hk, const char *name,
 				       const char *val)
 {
 	LONG ret;
+	TCHAR *_name, *_val;
+
+	_name = wpa_strdup_tchar(name);
+	if (_name == NULL)
+		return -1;
 
 	if (val == NULL) {
-		RegDeleteValue(hk, name);
+		RegDeleteValue(hk, _name);
+		free(_name);
 		return 0;
 	}
 
-	ret = RegSetValueEx(hk, name, 0, REG_SZ, val, strlen(val) + 1);
+	_val = wpa_strdup_tchar(val);
+	if (_val == NULL) {
+		free(_name);
+		return -1;
+	}
+	ret = RegSetValueEx(hk, _name, 0, REG_SZ, (BYTE *) _val, (strlen(val) + 1) * sizeof(TCHAR));
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_ERROR, "WINREG: Failed to set %s='%s': "
 			   "error %d", name, val, (int) GetLastError());
+		free(_name);
+		free(_val);
 		return -1;
 	}
 
+	free(_name);
+	free(_val);
 	return 0;
 }
 
@@ -446,27 +481,30 @@ static int wpa_config_write_global(struct wpa_config *config, HKEY hk)
 				    config->ctrl_interface);
 #endif /* CONFIG_CTRL_IFACE */
 
-	wpa_config_write_reg_dword(hk, "eapol_version", config->eapol_version,
+	wpa_config_write_reg_dword(hk, TEXT("eapol_version"),
+				   config->eapol_version,
 				   DEFAULT_EAPOL_VERSION);
-	wpa_config_write_reg_dword(hk, "ap_scan", config->ap_scan,
+	wpa_config_write_reg_dword(hk, TEXT("ap_scan"), config->ap_scan,
 				   DEFAULT_AP_SCAN);
-	wpa_config_write_reg_dword(hk, "fast_reauth", config->fast_reauth,
-				   DEFAULT_FAST_REAUTH);
-	wpa_config_write_reg_dword(hk, "dot11RSNAConfigPMKLifetime",
+	wpa_config_write_reg_dword(hk, TEXT("fast_reauth"),
+				   config->fast_reauth, DEFAULT_FAST_REAUTH);
+	wpa_config_write_reg_dword(hk, TEXT("dot11RSNAConfigPMKLifetime"),
 				   config->dot11RSNAConfigPMKLifetime, 0);
-	wpa_config_write_reg_dword(hk, "dot11RSNAConfigPMKReauthThreshold",
+	wpa_config_write_reg_dword(hk,
+				   TEXT("dot11RSNAConfigPMKReauthThreshold"),
 				   config->dot11RSNAConfigPMKReauthThreshold,
 				   0);
-	wpa_config_write_reg_dword(hk, "dot11RSNAConfigSATimeout",
+	wpa_config_write_reg_dword(hk, TEXT("dot11RSNAConfigSATimeout"),
 				   config->dot11RSNAConfigSATimeout, 0);
-	wpa_config_write_reg_dword(hk, "update_config", config->update_config,
+	wpa_config_write_reg_dword(hk, TEXT("update_config"),
+				   config->update_config,
 				   0);
 
 	return 0;
 }
 
 
-static int wpa_config_delete_subkeys(HKEY hk, const char *key)
+static int wpa_config_delete_subkeys(HKEY hk, const TCHAR *key)
 {
 	HKEY nhk;
 	int i, errors = 0;
@@ -474,8 +512,8 @@ static int wpa_config_delete_subkeys(HKEY hk, const char *key)
 
 	ret = RegOpenKeyEx(hk, key, 0, KEY_ENUMERATE_SUB_KEYS | DELETE, &nhk);
 	if (ret != ERROR_SUCCESS) {
-		wpa_printf(MSG_DEBUG, "WINREG: Could not open key '%s' for "
-			   "subkey deletion: error 0x%x (%d)", key,
+		wpa_printf(MSG_DEBUG, "WINREG: Could not open key '" TSTR
+			   "' for subkey deletion: error 0x%x (%d)", key,
 			   (unsigned int) ret, (int) GetLastError());
 		return 0;
 	}
@@ -499,7 +537,7 @@ static int wpa_config_delete_subkeys(HKEY hk, const char *key)
 
 		if (namelen >= 255)
 			namelen = 255 - 1;
-		name[namelen] = '\0';
+		name[namelen] = TEXT('\0');
 
 		ret = RegDeleteKey(nhk, name);
 		if (ret != ERROR_SUCCESS) {
@@ -669,9 +707,9 @@ static int wpa_config_write_network(HKEY hk, struct wpa_ssid *ssid, int id)
 	int i, errors = 0;
 	HKEY nhk, netw;
 	LONG ret;
-	char name[5];
+	TCHAR name[5];
 
-	ret = RegOpenKeyEx(hk, "networks", 0, KEY_CREATE_SUB_KEY, &nhk);
+	ret = RegOpenKeyEx(hk, TEXT("networks"), 0, KEY_CREATE_SUB_KEY, &nhk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_DEBUG, "WINREG: Could not open networks key "
 			   "for subkey addition: error 0x%x (%d)",
@@ -679,7 +717,11 @@ static int wpa_config_write_network(HKEY hk, struct wpa_ssid *ssid, int id)
 		return 0;
 	}
 
+#ifdef UNICODE
+	wsprintf(name, L"%04d", id);
+#else /* UNICODE */
 	snprintf(name, sizeof(name), "%04d", id);
+#endif /* UNICODE */
 	ret = RegCreateKeyEx(nhk, name, 0, NULL, 0, KEY_WRITE, NULL, &netw,
 			     NULL);
 	RegCloseKey(nhk);
@@ -745,6 +787,9 @@ static int wpa_config_write_network(HKEY hk, struct wpa_ssid *ssid, int id)
 	INT(mode);
 	INT(proactive_key_caching);
 	INT(disabled);
+	INT(stakey);
+	INT(peerkey);
+	STR(id_str);
 
 #undef STR
 #undef INT
@@ -760,9 +805,10 @@ static int wpa_config_write_blob(HKEY hk, struct wpa_config_blob *blob)
 {
 	HKEY bhk;
 	LONG ret;
+	TCHAR *name;
 
-	ret = RegCreateKeyEx(hk, "blobs", 0, NULL, 0, KEY_WRITE, NULL, &bhk,
-			     NULL);
+	ret = RegCreateKeyEx(hk, TEXT("blobs"), 0, NULL, 0, KEY_WRITE, NULL,
+			     &bhk, NULL);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_DEBUG, "WINREG: Could not add blobs key: "
 			   "error 0x%x (%d)",
@@ -770,15 +816,18 @@ static int wpa_config_write_blob(HKEY hk, struct wpa_config_blob *blob)
 		return -1;
 	}
 
-	ret = RegSetValueEx(bhk, blob->name, 0, REG_BINARY, blob->data,
+	name = wpa_strdup_tchar(blob->name);
+	ret = RegSetValueEx(bhk, name, 0, REG_BINARY, blob->data,
 			    blob->len);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_ERROR, "WINREG: Failed to set blob %s': "
 			   "error 0x%x (%d)", blob->name, (unsigned int) ret,
 			   (int) GetLastError());
 		RegCloseKey(bhk);
+		free(name);
 		return -1;
 	}
+	free(name);
 
 	RegCloseKey(bhk);
 
@@ -788,7 +837,7 @@ static int wpa_config_write_blob(HKEY hk, struct wpa_config_blob *blob)
 
 int wpa_config_write(const char *name, struct wpa_config *config)
 {
-	char buf[256];
+	TCHAR buf[256];
 	HKEY hk;
 	LONG ret;
 	int errors = 0;
@@ -798,7 +847,12 @@ int wpa_config_write(const char *name, struct wpa_config *config)
 
 	wpa_printf(MSG_DEBUG, "Writing configuration file '%s'", name);
 
-	snprintf(buf, sizeof(buf), KEY_PREFIX "\\configs\\%s", name);
+#ifdef UNICODE
+	_snwprintf(buf, 256, KEY_PREFIX TEXT("\\configs\\%S"), name);
+#else /* UNICODE */
+	snprintf(buf, 256, KEY_PREFIX TEXT("\\configs\\%s"), name);
+#endif /* UNICODE */
+
 	ret = RegOpenKeyEx(KEY_ROOT, buf, 0, KEY_SET_VALUE | DELETE, &hk);
 	if (ret != ERROR_SUCCESS) {
 		wpa_printf(MSG_ERROR, "Could not open wpa_supplicant "
@@ -813,13 +867,13 @@ int wpa_config_write(const char *name, struct wpa_config *config)
 		errors++;
 	}
 
-	wpa_config_delete_subkeys(hk, "networks");
+	wpa_config_delete_subkeys(hk, TEXT("networks"));
 	for (ssid = config->ssid, id = 0; ssid; ssid = ssid->next, id++) {
 		if (wpa_config_write_network(hk, ssid, id))
 			errors++;
 	}
 
-	RegDeleteKey(hk, "blobs");
+	RegDeleteKey(hk, TEXT("blobs"));
 	for (blob = config->blobs; blob; blob = blob->next) {
 		if (wpa_config_write_blob(hk, blob))
 			errors++;
