@@ -128,6 +128,21 @@ struct wpa_driver_associate_params {
 	 * mode - Operation mode (infra/ibss) IEEE80211_MODE_*
 	 */
 	int mode;
+
+	/**
+	 * wep_key - WEP keys for static WEP configuration
+	 */
+	const u8 *wep_key[4];
+
+	/**
+	 * wep_key_len - WEP key length for static WEP configuration
+	 */
+	size_t wep_key_len[4];
+
+	/**
+	 * wep_tx_keyidx - WEP TX key index for static WEP configuration
+	 */
+	int wep_tx_keyidx;
 };
 
 /**
