@@ -477,6 +477,10 @@ static int ndis_set_oid(struct wpa_driver_ndis_data *drv, unsigned int oid,
 	NDISUIO_SET_OID *o;
 	size_t buflen, reallen;
 	DWORD written;
+	char txt[50];
+
+	snprintf(txt, sizeof(txt), "NDIS: Set OID %08x", oid);
+	wpa_hexdump_key(MSG_MSGDUMP, txt, data, len);
 
 	buflen = sizeof(*o) + len;
 	reallen = buflen - sizeof(o->Data);
@@ -498,6 +502,10 @@ static int ndis_set_oid(struct wpa_driver_ndis_data *drv, unsigned int oid,
 #else /* CONFIG_USE_NDISUIO */
 	char *buf;
 	PACKET_OID_DATA *o;
+	char txt[50];
+
+	snprintf(txt, sizeof(txt), "NDIS: Set OID %08x", oid);
+	wpa_hexdump_key(MSG_MSGDUMP, txt, data, len);
 
 	buf = wpa_zalloc(sizeof(*o) + len);
 	if (buf == NULL)
@@ -784,6 +792,19 @@ static int wpa_driver_ndis_get_scan_results(void *priv,
 					bss->SupportedRates[j] & 0x7f;
 			}
 		}
+		if (((char *) bss->IEs) + bss->IELength  > (char *) b + blen) {
+			/*
+			 * Some NDIS drivers have been reported to include an
+			 * entry with an invalid IELength in scan results and
+			 * this has crashed wpa_supplicant, so validate the
+			 * returned value before using it.
+			 */
+			wpa_printf(MSG_DEBUG, "NDIS: skipped invalid scan "
+				   "result IE (BSSID=" MACSTR ") IELength=%d",
+				   MAC2STR(results[i].bssid),
+				   (int) bss->IELength);
+			break;
+		}
 		wpa_driver_ndis_get_ies(&results[i], bss->IEs, bss->IELength);
 		pos += bss->Length;
 		if (pos > (char *) b + blen)
@@ -849,7 +870,7 @@ static int wpa_driver_ndis_add_wep(struct wpa_driver_ndis_data *drv,
 	wep->KeyLength = key_len;
 	memcpy(wep->KeyMaterial, key, key_len);
 
-	wpa_hexdump_key(MSG_MSGDUMP, "NDIS: OIS_802_11_ADD_WEP",
+	wpa_hexdump_key(MSG_MSGDUMP, "NDIS: OID_802_11_ADD_WEP",
 			(char *) wep, len);
 	res = ndis_set_oid(drv, OID_802_11_ADD_WEP, (char *) wep, len);
 
@@ -918,7 +939,7 @@ static int wpa_driver_ndis_set_key(void *priv, wpa_alg alg, const u8 *addr,
 		memcpy(nkey->KeyMaterial, key, key_len);
 	}
 
-	wpa_hexdump_key(MSG_MSGDUMP, "NDIS: OIS_802_11_ADD_KEY",
+	wpa_hexdump_key(MSG_MSGDUMP, "NDIS: OID_802_11_ADD_KEY",
 			(char *) nkey, len);
 	res = ndis_set_oid(drv, OID_802_11_ADD_KEY, (char *) nkey, len);
 	free(nkey);
@@ -955,6 +976,23 @@ wpa_driver_ndis_associate(void *priv,
 		/* Try to continue anyway */
 	}
 
+	if (params->key_mgmt_suite == KEY_MGMT_NONE ||
+	    params->key_mgmt_suite == KEY_MGMT_802_1X_NO_WPA) {
+		/* Re-set WEP keys if static WEP configuration is used. */
+		u8 bcast[ETH_ALEN] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
+		int i;
+		for (i = 0; i < 4; i++) {
+			if (!params->wep_key[i])
+				continue;
+			wpa_printf(MSG_DEBUG, "NDIS: Re-setting static WEP "
+				   "key %d", i);
+			wpa_driver_ndis_set_key(drv, WPA_ALG_WEP, bcast, i,
+						i == params->wep_tx_keyidx,
+						NULL, 0, params->wep_key[i],
+						params->wep_key_len[i]);
+		}
+	}
+
 	if (params->wpa_ie == NULL || params->wpa_ie_len == 0) {
 		if (params->auth_alg & AUTH_ALG_SHARED_KEY) {
 			if (params->auth_alg & AUTH_ALG_OPEN_SYSTEM)
@@ -1567,8 +1605,7 @@ static void wpa_driver_ndis_get_capability(struct wpa_driver_ndis_data *drv)
 	size_t i;
 	NDIS_802_11_CAPABILITY *c;
 
-	drv->capa.flags = WPA_DRIVER_FLAGS_DRIVER_IE |
-		WPA_DRIVER_FLAGS_SET_KEYS_AFTER_ASSOC;
+	drv->capa.flags = WPA_DRIVER_FLAGS_DRIVER_IE;
 
 	len = ndis_get_oid(drv, OID_802_11_CAPABILITY, buf, sizeof(buf));
 	if (len < 0) {
@@ -1764,8 +1801,9 @@ static int wpa_driver_ndis_get_names(struct wpa_driver_ndis_data *drv)
 		   drv->adapter_desc);
 
 	return 0;
-#else /* CONFIG_USE_NDISUO */
-	PTSTR names, pos, pos2;
+#else /* CONFIG_USE_NDISUIO */
+	PTSTR _names;
+	char *names, *pos, *pos2;
 	ULONG len;
 	BOOLEAN res;
 #define MAX_ADAPTERS 32
@@ -1778,28 +1816,27 @@ static int wpa_driver_ndis_get_names(struct wpa_driver_ndis_data *drv)
 		   PacketGetVersion());
 
 	len = 8192;
-	names = wpa_zalloc(len);
-	if (names == NULL)
+	_names = wpa_zalloc(len);
+	if (_names == NULL)
 		return -1;
 
-	res = PacketGetAdapterNames(names, &len);
+	res = PacketGetAdapterNames(_names, &len);
 	if (!res && len > 8192) {
-		free(names);
-		names = wpa_zalloc(len);
-		if (names == NULL)
+		free(_names);
+		_names = wpa_zalloc(len);
+		if (_names == NULL)
 			return -1;
-		res = PacketGetAdapterNames(names, &len);
+		res = PacketGetAdapterNames(_names, &len);
 	}
 
 	if (!res) {
 		wpa_printf(MSG_ERROR, "NDIS: Failed to get adapter list "
 			   "(PacketGetAdapterNames)");
-		free(names);
+		free(_names);
 		return -1;
 	}
 
-	/* wpa_hexdump_ascii(MSG_DEBUG, "NDIS: AdapterNames", names, len); */
-
+	names = (char *) _names;
 	if (names[0] && names[1] == '\0' && names[2] && names[3] == '\0') {
 		wpa_printf(MSG_DEBUG, "NDIS: Looks like adapter names are in "
 			   "UNICODE");
@@ -1939,7 +1976,7 @@ static int wpa_driver_ndis_get_names(struct wpa_driver_ndis_data *drv)
 		   drv->adapter_desc);
 
 	return 0;
-#endif /* CONFIG_USE_NDISUO */
+#endif /* CONFIG_USE_NDISUIO */
 }
 
 
@@ -2015,7 +2052,7 @@ static int wpa_driver_ndis_set_wzc(struct wpa_driver_ndis_data *drv,
 	WCHAR *pos;
 	DWORD flags, i;
 
-	hm = LoadLibrary("wzcsapi.dll");
+	hm = LoadLibrary(TEXT("wzcsapi.dll"));
 	if (hm == NULL) {
 		wpa_printf(MSG_DEBUG, "NDIS: Failed to load wzcsapi.dll (%u) "
 			   "- WZC probably not running",
@@ -2279,9 +2316,6 @@ static void * wpa_driver_ndis_init(void *ctx, const char *ifname)
 	else if (strncmp(ifname, "\\DEVICE\\", 8) == 0)
 		ifname += 8;
 	strncpy(drv->ifname, ifname, sizeof(drv->ifname));
-#ifndef CONFIG_NDIS_EVENTS_INTEGRATED
-	drv->event_sock = -1;
-#endif /* CONFIG_NDIS_EVENTS_INTEGRATED */
 
 	if (wpa_driver_ndis_adapter_init(drv) < 0) {
 		free(drv);
@@ -2317,15 +2351,14 @@ static void * wpa_driver_ndis_init(void *ctx, const char *ifname)
 	eloop_register_timeout(1, 0, wpa_driver_ndis_poll_timeout, drv, NULL);
 
 #ifdef CONFIG_NDIS_EVENTS_INTEGRATED
-	drv->events = ndis_events_init(&drv->events_pipe, &drv->event_avail);
+	drv->events = ndis_events_init(&drv->events_pipe, &drv->event_avail,
+				       drv->ifname, drv->adapter_desc);
 	if (drv->events == NULL) {
 		wpa_driver_ndis_deinit(drv);
 		return NULL;
 	}
-	eloop_register_event(&drv->event_avail, sizeof(drv->event_avail),
+	eloop_register_event(drv->event_avail, sizeof(drv->event_avail),
 			     wpa_driver_ndis_event_pipe_cb, drv, NULL);
-#else /* CONFIG_NDIS_EVENTS_INTEGRATED */
-	wpa_driver_register_event_cb(drv);
 #endif /* CONFIG_NDIS_EVENTS_INTEGRATED */
 
 	/* Set mode here in case card was configured for ad-hoc mode
@@ -2357,7 +2390,7 @@ static void wpa_driver_ndis_deinit(void *priv)
 
 #ifdef CONFIG_NDIS_EVENTS_INTEGRATED
 	if (drv->events) {
-		eloop_unregister_event(&drv->event_avail,
+		eloop_unregister_event(drv->event_avail,
 				       sizeof(drv->event_avail));
 		ndis_events_deinit(drv->events);
 	}
@@ -2370,12 +2403,6 @@ static void wpa_driver_ndis_deinit(void *priv)
 		wpa_printf(MSG_DEBUG, "NDIS: failed to disassociate and turn "
 			   "radio off");
 	}
-#ifndef CONFIG_NDIS_EVENTS_INTEGRATED
-	if (drv->event_sock >= 0) {
-		eloop_unregister_read_sock(drv->event_sock);
-		close(drv->event_sock);
-	}
-#endif /* CONFIG_NDIS_EVENTS_INTEGRATED */
 
 	wpa_driver_ndis_adapter_close(drv);
 
