@@ -322,7 +322,8 @@ wpa_driver_wext_event_wireless_custom(void *ctx, char *custom)
 {
 	union wpa_event_data data;
 
-	wpa_printf(MSG_DEBUG, "Custom wireless event: '%s'", custom);
+	wpa_printf(MSG_MSGDUMP, "WEXT: Custom wireless event: '%s'",
+		   custom);
 
 	memset(&data, 0, sizeof(data));
 	/* Host AP driver */
@@ -653,13 +654,20 @@ static void wpa_driver_wext_event_rtm_newlink(struct wpa_driver_wext_data *drv,
 		return;
 	}
 
+	wpa_printf(MSG_DEBUG, "RTM_NEWLINK: operstate=%d ifi_flags=0x%x "
+		   "(%s%s%s%s)",
+		   drv->operstate, ifi->ifi_flags,
+		   (ifi->ifi_flags & IFF_UP) ? "[UP]" : "",
+		   (ifi->ifi_flags & IFF_RUNNING) ? "[RUNNING]" : "",
+		   (ifi->ifi_flags & IFF_LOWER_UP) ? "[LOWER_UP]" : "",
+		   (ifi->ifi_flags & IFF_DORMANT) ? "[DORMANT" : "");
 	/*
 	 * Some drivers send the association event before the operup event--in
 	 * this case, lifting operstate in wpa_driver_wext_set_operstate()
 	 * fails. This will hit us when wpa_supplicant does not need to do
 	 * IEEE 802.1X authentication
 	 */
-	if (drv->ifindex == ifi->ifi_index && drv->operstate == 1 &&
+	if (drv->operstate == 1 &&
 	    (ifi->ifi_flags & (IFF_LOWER_UP | IFF_DORMANT)) == IFF_LOWER_UP &&
 	    !(ifi->ifi_flags & IFF_RUNNING))
 		wpa_driver_wext_send_oper_ifla(drv, -1, IF_OPER_UP);
@@ -729,7 +737,9 @@ static void wpa_driver_wext_event_receive(int sock, void *eloop_ctx,
 	struct sockaddr_nl from;
 	socklen_t fromlen;
 	struct nlmsghdr *h;
+	int max_events = 10;
 
+try_again:
 	fromlen = sizeof(from);
 	left = recvfrom(sock, buf, sizeof(buf), MSG_DONTWAIT,
 			(struct sockaddr *) &from, &fromlen);
@@ -772,6 +782,17 @@ static void wpa_driver_wext_event_receive(int sock, void *eloop_ctx,
 		wpa_printf(MSG_DEBUG, "%d extra bytes in the end of netlink "
 			   "message", left);
 	}
+
+	if (--max_events > 0) {
+		/*
+		 * Try to receive all events in one eloop call in order to
+		 * limit race condition on cases where AssocInfo event, Assoc
+		 * event, and EAPOL frames are received more or less at the
+		 * same time. We want to process the event messages first
+		 * before starting EAPOL processing.
+		 */
+		goto try_again;
+	}
 }
 
 
@@ -1959,6 +1980,8 @@ int wpa_driver_wext_set_operstate(void *priv, int state)
 {
 	struct wpa_driver_wext_data *drv = priv;
 
+	wpa_printf(MSG_DEBUG, "%s: operstate %d->%d (%s)",
+		   __func__, drv->operstate, state, state ? "UP" : "DORMANT");
 	drv->operstate = state;
 	return wpa_driver_wext_send_oper_ifla(
 		drv, -1, state ? IF_OPER_UP : IF_OPER_DORMANT);
