@@ -1,6 +1,6 @@
 /*
- * WPA Supplicant / EAP-AKA (draft-arkko-pppext-eap-aka-12.txt)
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * EAP peer method: EAP-AKA (RFC 4187)
+ * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -12,46 +12,26 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "eap_i.h"
 #include "wpa_supplicant.h"
-#include "config_ssid.h"
 #include "crypto.h"
 #include "pcsc_funcs.h"
 #include "eap_sim_common.h"
 
-/* EAP-AKA Subtypes */
-#define EAP_AKA_SUBTYPE_CHALLENGE 1
-#define EAP_AKA_SUBTYPE_AUTHENTICATION_REJECT 2
-#define EAP_AKA_SUBTYPE_SYNCHRONIZATION_FAILURE 4
-#define EAP_AKA_SUBTYPE_IDENTITY 5
-#define EAP_AKA_SUBTYPE_NOTIFICATION 12
-#define EAP_AKA_SUBTYPE_REAUTHENTICATION 13
-#define EAP_AKA_SUBTYPE_CLIENT_ERROR 14
-
-/* AT_CLIENT_ERROR_CODE error codes */
-#define EAP_AKA_UNABLE_TO_PROCESS_PACKET 0
-
-#define AKA_AUTS_LEN 14
-#define RES_MAX_LEN 16
-#define IK_LEN 16
-#define CK_LEN 16
-#define EAP_AKA_MAX_FAST_REAUTHS 1000
 
 struct eap_aka_data {
-	u8 ik[IK_LEN], ck[CK_LEN], res[RES_MAX_LEN];
+	u8 ik[EAP_AKA_IK_LEN], ck[EAP_AKA_CK_LEN], res[EAP_AKA_RES_MAX_LEN];
 	size_t res_len;
 	u8 nonce_s[EAP_SIM_NONCE_S_LEN];
 	u8 mk[EAP_SIM_MK_LEN];
 	u8 k_aut[EAP_SIM_K_AUT_LEN];
 	u8 k_encr[EAP_SIM_K_ENCR_LEN];
 	u8 msk[EAP_SIM_KEYING_DATA_LEN];
-	u8 rand[AKA_RAND_LEN], autn[AKA_AUTN_LEN];
-	u8 auts[AKA_AUTS_LEN];
+	u8 rand[EAP_AKA_RAND_LEN], autn[EAP_AKA_AUTN_LEN];
+	u8 auts[EAP_AKA_AUTS_LEN];
 
 	int num_id_req, num_notification;
 	u8 *pseudonym;
@@ -69,10 +49,9 @@ struct eap_aka_data {
 static void * eap_aka_init(struct eap_sm *sm)
 {
 	struct eap_aka_data *data;
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
 		return NULL;
-	memset(data, 0, sizeof(*data));
 
 	data->state = CONTINUE;
 
@@ -102,45 +81,34 @@ static int eap_aka_umts_auth(struct eap_sm *sm, struct eap_aka_data *data)
 #else /* PCSC_FUNCS */
 	/* These hardcoded Kc and SRES values are used for testing.
 	 * Could consider making them configurable. */
-	memset(data->res, '2', RES_MAX_LEN);
-	data->res_len = 16;
-	memset(data->ik, '3', IK_LEN);
-	memset(data->ck, '4', CK_LEN);
+	memset(data->res, '2', EAP_AKA_RES_MAX_LEN);
+	data->res_len = EAP_AKA_RES_MAX_LEN;
+	memset(data->ik, '3', EAP_AKA_IK_LEN);
+	memset(data->ck, '4', EAP_AKA_CK_LEN);
 	{
-		u8 autn[AKA_AUTN_LEN];
-		memset(autn, '1', AKA_AUTN_LEN);
-		if (memcmp(autn, data->autn, AKA_AUTN_LEN) != 0) {
+		u8 autn[EAP_AKA_AUTN_LEN];
+		memset(autn, '1', EAP_AKA_AUTN_LEN);
+		if (memcmp(autn, data->autn, EAP_AKA_AUTN_LEN) != 0) {
 			wpa_printf(MSG_WARNING, "EAP-AKA: AUTN did not match "
 				   "with expected value");
 			return -1;
 		}
 	}
+#if 0
+	{
+		static int test_resync = 1;
+		if (test_resync) {
+			/* Test Resynchronization */
+			test_resync = 0;
+			return -2;
+		}
+	}
+#endif
 	return 0;
 #endif /* PCSC_FUNCS */
 }
 
 
-static void eap_aka_derive_mk(struct eap_aka_data *data,
-			      const u8 *identity, size_t identity_len)
-{
-	const u8 *addr[3];
-	size_t len[3];
-
-	addr[0] = identity;
-	len[0] = identity_len;
-	addr[1] = data->ik;
-	len[1] = IK_LEN;
-	addr[2] = data->ck;
-	len[2] = CK_LEN;
-
-	/* MK = SHA1(Identity|IK|CK) */
-	sha1_vector(3, addr, len, data->mk);
-	wpa_hexdump_key(MSG_DEBUG, "EAP-AKA: IK", data->ik, IK_LEN);
-	wpa_hexdump_key(MSG_DEBUG, "EAP-AKA: CK", data->ck, CK_LEN);
-	wpa_hexdump_key(MSG_DEBUG, "EAP-AKA: MK", data->mk, EAP_SIM_MK_LEN);
-}
-
-
 #define CLEAR_PSEUDONYM	0x01
 #define CLEAR_REAUTH_ID	0x02
 #define CLEAR_EAP_ID	0x04
@@ -210,7 +178,7 @@ static int eap_aka_learn_ids(struct eap_aka_data *data,
 }
 
 
-static u8 * eap_aka_client_error(struct eap_sm *sm, struct eap_aka_data *data,
+static u8 * eap_aka_client_error(struct eap_aka_data *data,
 				 const struct eap_hdr *req,
 				 size_t *respDataLen, int err)
 {
@@ -227,8 +195,7 @@ static u8 * eap_aka_client_error(struct eap_sm *sm, struct eap_aka_data *data,
 }
 
 
-static u8 * eap_aka_authentication_reject(struct eap_sm *sm,
-					  struct eap_aka_data *data,
+static u8 * eap_aka_authentication_reject(struct eap_aka_data *data,
 					  const struct eap_hdr *req,
 					  size_t *respDataLen)
 {
@@ -247,14 +214,12 @@ static u8 * eap_aka_authentication_reject(struct eap_sm *sm,
 }
 
 
-static u8 * eap_aka_synchronization_failure(struct eap_sm *sm,
-					    struct eap_aka_data *data,
+static u8 * eap_aka_synchronization_failure(struct eap_aka_data *data,
 					    const struct eap_hdr *req,
 					    size_t *respDataLen)
 {
 	struct eap_sim_msg *msg;
 
-	data->state = FAILURE;
 	data->num_id_req = 0;
 	data->num_notification = 0;
 
@@ -264,7 +229,8 @@ static u8 * eap_aka_synchronization_failure(struct eap_sm *sm,
 			       EAP_TYPE_AKA,
 			       EAP_AKA_SUBTYPE_SYNCHRONIZATION_FAILURE);
 	wpa_printf(MSG_DEBUG, "   AT_AUTS");
-	eap_sim_msg_add_full(msg, EAP_SIM_AT_AUTS, data->auts, AKA_AUTS_LEN);
+	eap_sim_msg_add_full(msg, EAP_SIM_AT_AUTS, data->auts,
+			     EAP_AKA_AUTS_LEN);
 	return eap_sim_msg_finish(msg, respDataLen, NULL, NULL, 0);
 }
 
@@ -275,8 +241,7 @@ static u8 * eap_aka_response_identity(struct eap_sm *sm,
 				      size_t *respDataLen,
 				      enum eap_sim_id_req id_req)
 {
-	struct wpa_ssid *config = eap_get_config(sm);
-	u8 *identity = NULL;
+	const u8 *identity = NULL;
 	size_t identity_len = 0;
 	struct eap_sim_msg *msg;
 
@@ -290,11 +255,12 @@ static u8 * eap_aka_response_identity(struct eap_sm *sm,
 		identity = data->pseudonym;
 		identity_len = data->pseudonym_len;
 		eap_aka_clear_identities(data, CLEAR_REAUTH_ID);
-	} else if (id_req != NO_ID_REQ && config && config->identity) {
-		identity = config->identity;
-		identity_len = config->identity_len;
-		eap_aka_clear_identities(data,
-					 CLEAR_PSEUDONYM | CLEAR_REAUTH_ID);
+	} else if (id_req != NO_ID_REQ) {
+		identity = eap_get_config_identity(sm, &identity_len);
+		if (identity) {
+			eap_aka_clear_identities(data, CLEAR_PSEUDONYM |
+						 CLEAR_REAUTH_ID);
+		}
 	}
 	if (id_req != NO_ID_REQ)
 		eap_aka_clear_identities(data, CLEAR_EAP_ID);
@@ -315,8 +281,7 @@ static u8 * eap_aka_response_identity(struct eap_sm *sm,
 }
 
 
-static u8 * eap_aka_response_challenge(struct eap_sm *sm,
-				       struct eap_aka_data *data,
+static u8 * eap_aka_response_challenge(struct eap_aka_data *data,
 				       const struct eap_hdr *req,
 				       size_t *respDataLen)
 {
@@ -335,8 +300,7 @@ static u8 * eap_aka_response_challenge(struct eap_sm *sm,
 }
 
 
-static u8 * eap_aka_response_reauth(struct eap_sm *sm,
-				    struct eap_aka_data *data,
+static u8 * eap_aka_response_reauth(struct eap_aka_data *data,
 				    const struct eap_hdr *req,
 				    size_t *respDataLen, int counter_too_small)
 {
@@ -375,8 +339,7 @@ static u8 * eap_aka_response_reauth(struct eap_sm *sm,
 }
 
 
-static u8 * eap_aka_response_notification(struct eap_sm *sm,
-					  struct eap_aka_data *data,
+static u8 * eap_aka_response_notification(struct eap_aka_data *data,
 					  const struct eap_hdr *req,
 					  size_t *respDataLen,
 					  u16 notification)
@@ -417,7 +380,6 @@ static u8 * eap_aka_response_notification(struct eap_sm *sm,
 static u8 * eap_aka_process_identity(struct eap_sm *sm,
 				     struct eap_aka_data *data,
 				     const struct eap_hdr *req,
-				     size_t reqDataLen,
 				     size_t *respDataLen,
 				     struct eap_sim_attrs *attr)
 {
@@ -448,7 +410,7 @@ static u8 * eap_aka_process_identity(struct eap_sm *sm,
 	if (id_error) {
 		wpa_printf(MSG_INFO, "EAP-AKA: Too many ID requests "
 			   "used within one authentication");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -464,8 +426,7 @@ static u8 * eap_aka_process_challenge(struct eap_sm *sm,
 				      size_t *respDataLen,
 				      struct eap_sim_attrs *attr)
 {
-	struct wpa_ssid *config = eap_get_config(sm);
-	u8 *identity;
+	const u8 *identity;
 	size_t identity_len;
 	int res;
 	struct eap_sim_attrs eattr;
@@ -478,26 +439,24 @@ static u8 * eap_aka_process_challenge(struct eap_sm *sm,
 			   !attr->mac ? " AT_MAC" : "",
 			   !attr->rand ? " AT_RAND" : "",
 			   !attr->autn ? " AT_AUTN" : "");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
-	memcpy(data->rand, attr->rand, AKA_RAND_LEN);
-	memcpy(data->autn, attr->autn, AKA_AUTN_LEN);
+	memcpy(data->rand, attr->rand, EAP_AKA_RAND_LEN);
+	memcpy(data->autn, attr->autn, EAP_AKA_AUTN_LEN);
 
 	res = eap_aka_umts_auth(sm, data);
 	if (res == -1) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: UMTS authentication "
 			   "failed (AUTN)");
-		return eap_aka_authentication_reject(sm, data, req,
-						     respDataLen);
+		return eap_aka_authentication_reject(data, req, respDataLen);
 	} else if (res == -2) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: UMTS authentication "
 			   "failed (AUTN seq# -> AUTS)");
-		return eap_aka_synchronization_failure(sm, data, req,
-						       respDataLen);
+		return eap_aka_synchronization_failure(data, req, respDataLen);
 	} else if (res) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: UMTS authentication failed");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 	if (data->last_eap_identity) {
@@ -506,19 +465,18 @@ static u8 * eap_aka_process_challenge(struct eap_sm *sm,
 	} else if (data->pseudonym) {
 		identity = data->pseudonym;
 		identity_len = data->pseudonym_len;
-	} else {
-		identity = config->identity;
-		identity_len = config->identity_len;
-	}
+	} else
+		identity = eap_get_config_identity(sm, &identity_len);
 	wpa_hexdump_ascii(MSG_DEBUG, "EAP-AKA: Selected identity for MK "
 			  "derivation", identity, identity_len);
-	eap_aka_derive_mk(data, identity, identity_len);
+	eap_aka_derive_mk(identity, identity_len, data->ik, data->ck,
+			  data->mk);
 	eap_sim_derive_keys(data->mk, data->k_encr, data->k_aut, data->msk);
 	if (eap_sim_verify_mac(data->k_aut, (const u8 *) req, reqDataLen,
 			       attr->mac, (u8 *) "", 0)) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Challenge message "
 			   "used invalid AT_MAC");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -535,7 +493,7 @@ static u8 * eap_aka_process_challenge(struct eap_sm *sm,
 					       &eattr, 0);
 		if (decrypted == NULL) {
 			return eap_aka_client_error(
-				sm, data, req, respDataLen,
+				data, req, respDataLen,
 				EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 		}
 		eap_aka_learn_ids(data, &eattr);
@@ -547,17 +505,15 @@ static u8 * eap_aka_process_challenge(struct eap_sm *sm,
 
 	data->num_id_req = 0;
 	data->num_notification = 0;
-	/* draft-arkko-pppext-eap-aka-12.txt specifies that counter
-	 * is initialized to one after fullauth, but initializing it to
-	 * zero makes it easier to implement reauth verification. */
+	/* RFC 4187 specifies that counter is initialized to one after
+	 * fullauth, but initializing it to zero makes it easier to implement
+	 * reauth verification. */
 	data->counter = 0;
-	return eap_aka_response_challenge(sm, data, req, respDataLen);
+	return eap_aka_response_challenge(data, req, respDataLen);
 }
 
 
 static int eap_aka_process_notification_reauth(struct eap_aka_data *data,
-					       const struct eap_hdr *req,
-					       size_t reqDataLen,
 					       struct eap_sim_attrs *attr)
 {
 	struct eap_sim_attrs eattr;
@@ -578,7 +534,7 @@ static int eap_aka_process_notification_reauth(struct eap_aka_data *data,
 		return -1;
 	}
 
-	if (eattr.counter != data->counter) {
+	if (eattr.counter < 0 || (size_t) eattr.counter != data->counter) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Counter in notification "
 			   "message does not match with counter in reauth "
 			   "message");
@@ -610,7 +566,7 @@ static int eap_aka_process_notification_auth(struct eap_aka_data *data,
 	}
 
 	if (data->reauth &&
-	    eap_aka_process_notification_reauth(data, req, reqDataLen, attr)) {
+	    eap_aka_process_notification_reauth(data, attr)) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Invalid notification "
 			   "message after reauth");
 		return -1;
@@ -631,20 +587,20 @@ static u8 * eap_aka_process_notification(struct eap_sm *sm,
 	if (data->num_notification > 0) {
 		wpa_printf(MSG_INFO, "EAP-AKA: too many notification "
 			   "rounds (only one allowed)");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 	data->num_notification++;
 	if (attr->notification == -1) {
 		wpa_printf(MSG_INFO, "EAP-AKA: no AT_NOTIFICATION in "
 			   "Notification message");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
 	if ((attr->notification & 0x4000) == 0 &&
 	    eap_aka_process_notification_auth(data, req, reqDataLen, attr)) {
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -652,7 +608,7 @@ static u8 * eap_aka_process_notification(struct eap_sm *sm,
 	if (attr->notification >= 0 && attr->notification < 32768) {
 		data->state = FAILURE;
 	}
-	return eap_aka_response_notification(sm, data, req, respDataLen,
+	return eap_aka_response_notification(data, req, respDataLen,
 					     attr->notification);
 }
 
@@ -672,7 +628,7 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 	if (data->reauth_id == NULL) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Server is trying "
 			   "reauthentication, but no reauth_id available");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -681,14 +637,14 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 			       attr->mac, (u8 *) "", 0)) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Reauthentication "
 			   "did not have valid AT_MAC");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
 	if (attr->encr_data == NULL || attr->iv == NULL) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Reauthentication "
 			   "message did not include encrypted data");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -698,7 +654,7 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 	if (decrypted == NULL) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Failed to parse encrypted "
 			   "data from reauthentication message");
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
@@ -707,11 +663,11 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 			   !eattr.nonce_s ? " AT_NONCE_S" : "",
 			   eattr.counter < 0 ? " AT_COUNTER" : "");
 		free(decrypted);
-		return eap_aka_client_error(sm, data, req, respDataLen,
+		return eap_aka_client_error(data, req, respDataLen,
 					    EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 	}
 
-	if (eattr.counter <= data->counter) {
+	if (eattr.counter < 0 || (size_t) eattr.counter <= data->counter) {
 		wpa_printf(MSG_INFO, "EAP-AKA: (encr) Invalid counter "
 			   "(%d <= %d)", eattr.counter, data->counter);
 		data->counter_too_small = eattr.counter;
@@ -726,7 +682,7 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 		data->reauth_id = NULL;
 		data->reauth_id_len = 0;
 		free(decrypted);
-		return eap_aka_response_reauth(sm, data, req, respDataLen, 1);
+		return eap_aka_response_reauth(data, req, respDataLen, 1);
 	}
 	data->counter = eattr.counter;
 
@@ -751,7 +707,7 @@ static u8 * eap_aka_process_reauthentication(struct eap_sm *sm,
 		eap_aka_clear_identities(data, CLEAR_REAUTH_ID | CLEAR_EAP_ID);
 	}
 	free(decrypted);
-	return eap_aka_response_reauth(sm, data, req, respDataLen, 0);
+	return eap_aka_response_reauth(data, req, respDataLen, 0);
 }
 
 
@@ -761,7 +717,6 @@ static u8 * eap_aka_process(struct eap_sm *sm, void *priv,
 			    size_t *respDataLen)
 {
 	struct eap_aka_data *data = priv;
-	struct wpa_ssid *config = eap_get_config(sm);
 	const struct eap_hdr *req;
 	u8 subtype, *res;
 	const u8 *pos;
@@ -769,14 +724,15 @@ static u8 * eap_aka_process(struct eap_sm *sm, void *priv,
 	size_t len;
 
 	wpa_hexdump(MSG_DEBUG, "EAP-AKA: EAP data", reqData, reqDataLen);
-	if (config == NULL || config->identity == NULL) {
+	if (eap_get_config_identity(sm, &len) == NULL) {
 		wpa_printf(MSG_INFO, "EAP-AKA: Identity not configured");
-		eap_sm_request_identity(sm, config);
+		eap_sm_request_identity(sm);
 		ret->ignore = TRUE;
 		return NULL;
 	}
 
-	pos = eap_hdr_validate(EAP_TYPE_AKA, reqData, reqDataLen, &len);
+	pos = eap_hdr_validate(EAP_VENDOR_IETF, EAP_TYPE_AKA,
+			       reqData, reqDataLen, &len);
 	if (pos == NULL || len < 1) {
 		ret->ignore = TRUE;
 		return NULL;
@@ -785,7 +741,7 @@ static u8 * eap_aka_process(struct eap_sm *sm, void *priv,
 	len = be_to_host16(req->length);
 
 	ret->ignore = FALSE;
-	ret->methodState = METHOD_CONT;
+	ret->methodState = METHOD_MAY_CONT;
 	ret->decision = DECISION_FAIL;
 	ret->allowNotifications = TRUE;
 
@@ -794,14 +750,14 @@ static u8 * eap_aka_process(struct eap_sm *sm, void *priv,
 	pos += 2; /* Reserved */
 
 	if (eap_sim_parse_attr(pos, reqData + len, &attr, 1, 0)) {
-		res = eap_aka_client_error(sm, data, req, respDataLen,
+		res = eap_aka_client_error(data, req, respDataLen,
 					   EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 		goto done;
 	}
 
 	switch (subtype) {
 	case EAP_AKA_SUBTYPE_IDENTITY:
-		res = eap_aka_process_identity(sm, data, req, len,
+		res = eap_aka_process_identity(sm, data, req,
 					       respDataLen, &attr);
 		break;
 	case EAP_AKA_SUBTYPE_CHALLENGE:
@@ -818,12 +774,12 @@ static u8 * eap_aka_process(struct eap_sm *sm, void *priv,
 		break;
 	case EAP_AKA_SUBTYPE_CLIENT_ERROR:
 		wpa_printf(MSG_DEBUG, "EAP-AKA: subtype Client-Error");
-		res = eap_aka_client_error(sm, data, req, respDataLen,
+		res = eap_aka_client_error(data, req, respDataLen,
 					   EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 		break;
 	default:
 		wpa_printf(MSG_DEBUG, "EAP-AKA: Unknown subtype=%d", subtype);
-		res = eap_aka_client_error(sm, data, req, respDataLen,
+		res = eap_aka_client_error(data, req, respDataLen,
 					   EAP_AKA_UNABLE_TO_PROCESS_PACKET);
 		break;
 	}
@@ -833,7 +789,7 @@ done:
 		ret->decision = DECISION_FAIL;
 		ret->methodState = METHOD_DONE;
 	} else if (data->state == SUCCESS) {
-		ret->decision = DECISION_UNCOND_SUCC;
+		ret->decision = DECISION_COND_SUCC;
 		ret->methodState = METHOD_DONE;
 	}
 
@@ -914,17 +870,28 @@ static u8 * eap_aka_getKey(struct eap_sm *sm, void *priv, size_t *len)
 }
 
 
-const struct eap_method eap_method_aka =
+int eap_peer_aka_register(void)
 {
-	.method = EAP_TYPE_AKA,
-	.name = "AKA",
-	.init = eap_aka_init,
-	.deinit = eap_aka_deinit,
-	.process = eap_aka_process,
-	.isKeyAvailable = eap_aka_isKeyAvailable,
-	.getKey = eap_aka_getKey,
-	.has_reauth_data = eap_aka_has_reauth_data,
-	.deinit_for_reauth = eap_aka_deinit_for_reauth,
-	.init_for_reauth = eap_aka_init_for_reauth,
-	.get_identity = eap_aka_get_identity,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_peer_method_alloc(EAP_PEER_METHOD_INTERFACE_VERSION,
+				    EAP_VENDOR_IETF, EAP_TYPE_AKA, "AKA");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_aka_init;
+	eap->deinit = eap_aka_deinit;
+	eap->process = eap_aka_process;
+	eap->isKeyAvailable = eap_aka_isKeyAvailable;
+	eap->getKey = eap_aka_getKey;
+	eap->has_reauth_data = eap_aka_has_reauth_data;
+	eap->deinit_for_reauth = eap_aka_deinit_for_reauth;
+	eap->init_for_reauth = eap_aka_init_for_reauth;
+	eap->get_identity = eap_aka_get_identity;
+
+	ret = eap_peer_method_register(eap);
+	if (ret)
+		eap_peer_method_free(eap);
+	return ret;
+}
