@@ -469,6 +469,7 @@ static void eap_fast_write(char **buf, char **pos, size_t *buf_len,
 			   size_t len, int txt)
 {
 	size_t i, need;
+	int ret;
 
 	if (data == NULL || *buf == NULL)
 		return;
@@ -488,18 +489,33 @@ static void eap_fast_write(char **buf, char **pos, size_t *buf_len,
 		*buf_len += need;
 	}
 
-	*pos += snprintf(*pos, *buf + *buf_len - *pos, "%s=", field);
+	ret = snprintf(*pos, *buf + *buf_len - *pos, "%s=", field);
+	if (ret < 0 || ret >= *buf + *buf_len - *pos)
+		return;
+	*pos += ret;
 	*pos += wpa_snprintf_hex(*pos, *buf + *buf_len - *pos, data, len);
-	*pos += snprintf(*pos, *buf + *buf_len - *pos, "\n");
+	ret = snprintf(*pos, *buf + *buf_len - *pos, "\n");
+	if (ret < 0 || ret >= *buf + *buf_len - *pos)
+		return;
+	*pos += ret;
 
 	if (txt) {
-		*pos += snprintf(*pos, *buf + *buf_len - *pos,
-				 "%s-txt=", field);
+		ret = snprintf(*pos, *buf + *buf_len - *pos,
+			       "%s-txt=", field);
+		if (ret < 0 || ret >= *buf + *buf_len - *pos)
+			return;
+		*pos += ret;
 		for (i = 0; i < len; i++) {
-			*pos += snprintf(*pos, *buf + *buf_len - *pos,
-					 "%c", data[i]);
+			ret = snprintf(*pos, *buf + *buf_len - *pos,
+				       "%c", data[i]);
+			if (ret < 0 || ret >= *buf + *buf_len - *pos)
+				return;
+			*pos += ret;
 		}
-		*pos += snprintf(*pos, *buf + *buf_len - *pos, "\n");
+		ret = snprintf(*pos, *buf + *buf_len - *pos, "\n");
+		if (ret < 0 || ret >= *buf + *buf_len - *pos)
+			return;
+		*pos += ret;
 	}
 }
 
@@ -509,7 +525,7 @@ static int eap_fast_save_pac(struct eap_sm *sm, struct eap_fast_data *data,
 {
 	FILE *f;
 	struct eap_fast_pac *pac;
-	int count = 0;
+	int count = 0, ret;
 	char *buf, *pos;
 	size_t buf_len;
 
@@ -521,11 +537,17 @@ static int eap_fast_save_pac(struct eap_sm *sm, struct eap_fast_data *data,
 	if (buf == NULL)
 		return -1;
 
-	pos += snprintf(pos, buf + buf_len - pos, "%s\n", pac_file_hdr);
+	ret = snprintf(pos, buf + buf_len - pos, "%s\n", pac_file_hdr);
+	if (ret < 0 || ret >= buf + buf_len - pos)
+		return -1;
+	pos += ret;
 
 	pac = data->pac;
 	while (pac) {
-		pos += snprintf(pos, buf + buf_len - pos, "START\n");
+		ret = snprintf(pos, buf + buf_len - pos, "START\n");
+		if (ret < 0 || ret >= buf + buf_len - pos)
+			return -1;
+		pos += ret;
 		eap_fast_write(&buf, &pos, &buf_len, "PAC-Key", pac->pac_key,
 			       EAP_FAST_PAC_KEY_LEN, 0);
 		eap_fast_write(&buf, &pos, &buf_len, "PAC-Opaque",
@@ -538,7 +560,10 @@ static int eap_fast_save_pac(struct eap_sm *sm, struct eap_fast_data *data,
 			       pac->i_id_len, 1);
 		eap_fast_write(&buf, &pos, &buf_len, "A-ID-Info",
 			       pac->a_id_info, pac->a_id_info_len, 1);
-		pos += snprintf(pos, buf + buf_len - pos, "END\n");
+		ret = snprintf(pos, buf + buf_len - pos, "END\n");
+		if (ret < 0 || ret >= buf + buf_len - pos)
+			return -1;
+		pos += ret;
 		count++;
 		pac = pac->next;
 
@@ -1851,6 +1876,7 @@ static u8 * eap_fast_process(struct eap_sm *sm, void *priv,
 			}
 			free(tlv);
 		} else {
+			u8 ciphers[2];
 			if (!data->provisioning_allowed) {
 				wpa_printf(MSG_DEBUG, "EAP-FAST: No PAC found "
 					   "and provisioning disabled");
@@ -1858,8 +1884,11 @@ static u8 * eap_fast_process(struct eap_sm *sm, void *priv,
 			}
 			wpa_printf(MSG_DEBUG, "EAP-FAST: No PAC found - "
 				   "starting provisioning");
-			if (tls_connection_set_anon_dh(sm->ssl_ctx,
-						       data->ssl.conn)) {
+			ciphers[0] = TLS_CIPHER_ANON_DH_AES128_SHA;
+			ciphers[1] = TLS_CIPHER_NONE;
+			if (tls_connection_set_cipher_list(sm->ssl_ctx,
+							   data->ssl.conn,
+							   ciphers)) {
 				wpa_printf(MSG_INFO, "EAP-FAST: Could not "
 					   "configure anonymous DH for TLS "
 					   "connection");
@@ -1938,6 +1967,9 @@ static void * eap_fast_init_for_reauth(struct eap_sm *sm, void *priv)
 		free(data);
 		return NULL;
 	}
+	if (data->phase2_priv && data->phase2_method &&
+	    data->phase2_method->init_for_reauth)
+		data->phase2_method->init_for_reauth(sm, data->phase2_priv);
 	data->phase2_success = 0;
 	data->resuming = 1;
 	data->provisioning = 0;
@@ -1950,13 +1982,16 @@ static int eap_fast_get_status(struct eap_sm *sm, void *priv, char *buf,
 			       size_t buflen, int verbose)
 {
 	struct eap_fast_data *data = priv;
-	int len;
+	int len, ret;
 
 	len = eap_tls_status(sm, &data->ssl, buf, buflen, verbose);
 	if (data->phase2_method) {
-		len += snprintf(buf + len, buflen - len,
-				"EAP-FAST Phase2 method=%s\n",
-				data->phase2_method->name);
+		ret = snprintf(buf + len, buflen - len,
+			       "EAP-FAST Phase2 method=%s\n",
+			       data->phase2_method->name);
+		if (ret < 0 || (size_t) ret >= buflen - len)
+			return len;
+		len += ret;
 	}
 	return len;
 }
