@@ -344,7 +344,7 @@ const u8 * eap_tls_data_reassemble(
 		}
 	} else {
 		data->tls_in_left = 0;
-		data->tls_in = malloc(in_len);
+		data->tls_in = malloc(in_len ? in_len : 1);
 		if (data->tls_in == NULL)
 			return NULL;
 		memcpy(data->tls_in, in_data, in_len);
@@ -357,11 +357,14 @@ const u8 * eap_tls_data_reassemble(
 
 
 static int eap_tls_process_input(struct eap_sm *sm, struct eap_ssl_data *data,
-				 const u8 *in_data, size_t in_len)
+				 const u8 *in_data, size_t in_len,
+				 u8 **out_data, size_t *out_len)
 {
 	const u8 *msg;
 	size_t msg_len;
 	int need_more_input;
+	u8 *appl_data;
+	size_t appl_data_len;
 
 	msg = eap_tls_data_reassemble(sm, data, in_data, in_len,
 				      &msg_len, &need_more_input);
@@ -376,15 +379,29 @@ static int eap_tls_process_input(struct eap_sm *sm, struct eap_ssl_data *data,
 		free(data->tls_out);
 		WPA_ASSERT(data->tls_out == NULL);
 	}
+	appl_data = NULL;
 	data->tls_out = tls_connection_handshake(sm->ssl_ctx, data->conn,
 						 msg, msg_len,
-						 &data->tls_out_len);
+						 &data->tls_out_len,
+						 &appl_data, &appl_data_len);
 
 	/* Clear reassembled input data (if the buffer was needed). */
 	data->tls_in_left = data->tls_in_total = data->tls_in_len = 0;
 	free(data->tls_in);
 	data->tls_in = NULL;
 
+	if (appl_data &&
+	    tls_connection_established(sm->ssl_ctx, data->conn) &&
+	    !tls_connection_get_failed(sm->ssl_ctx, data->conn)) {
+		wpa_hexdump_key(MSG_MSGDUMP, "SSL: Application data",
+				appl_data, appl_data_len);
+		*out_data = appl_data;
+		*out_len = appl_data_len;
+		return 2;
+	}
+
+	free(appl_data);
+
 	return 0;
 }
 
@@ -483,7 +500,8 @@ int eap_tls_process_helper(struct eap_sm *sm, struct eap_ssl_data *data,
 	if (data->tls_out_len == 0) {
 		/* No more data to send out - expect to receive more data from
 		 * the AS. */
-		int res = eap_tls_process_input(sm, data, in_data, in_len);
+		int res = eap_tls_process_input(sm, data, in_data, in_len,
+						out_data, out_len);
 		if (res)
 			return res;
 	}
@@ -572,11 +590,14 @@ int eap_tls_status(struct eap_sm *sm, struct eap_ssl_data *data, char *buf,
 		   size_t buflen, int verbose)
 {
 	char name[128];
-	int len = 0;
+	int len = 0, ret;
 
 	if (tls_get_cipher(sm->ssl_ctx, data->conn, name, sizeof(name)) == 0) {
-		len += snprintf(buf + len, buflen - len,
-				"EAP TLS cipher=%s\n", name);
+		ret = snprintf(buf + len, buflen - len,
+			       "EAP TLS cipher=%s\n", name);
+		if (ret < 0 || (size_t) ret >= buflen - len)
+			return len;
+		len += ret;
 	}
 
 	return len;
