@@ -1579,6 +1579,19 @@ static u8 * eap_ttls_process(struct eap_sm *sm, void *priv,
 			}
 			data->resuming = 0;
 		}
+
+		if (res == 2) {
+			/*
+			 * Application data included in the handshake message.
+			 */
+			free(data->pending_phase2_req);
+			data->pending_phase2_req = resp;
+			data->pending_phase2_req_len = *respDataLen;
+			resp = NULL;
+			*respDataLen = 0;
+			res = eap_ttls_decrypt(sm, data, ret, req, pos, left,
+					       &resp, respDataLen);
+		}
 	}
 
 	if (data->ttls_version == 0 && ret->methodState == METHOD_DONE) {
@@ -1631,6 +1644,9 @@ static void * eap_ttls_init_for_reauth(struct eap_sm *sm, void *priv)
 		free(data);
 		return NULL;
 	}
+	if (data->phase2_priv && data->phase2_method &&
+	    data->phase2_method->init_for_reauth)
+		data->phase2_method->init_for_reauth(sm, data->phase2_priv);
 	data->phase2_start = 0;
 	data->phase2_success = 0;
 	data->resuming = 1;
@@ -1643,30 +1659,39 @@ static int eap_ttls_get_status(struct eap_sm *sm, void *priv, char *buf,
 			       size_t buflen, int verbose)
 {
 	struct eap_ttls_data *data = priv;
-	int len;
+	int len, ret;
 
 	len = eap_tls_status(sm, &data->ssl, buf, buflen, verbose);
-	len += snprintf(buf + len, buflen - len, "EAP-TTLSv%d Phase2 method=",
-			data->ttls_version);
+	ret = snprintf(buf + len, buflen - len, "EAP-TTLSv%d Phase2 method=",
+		       data->ttls_version);
+	if (ret < 0 || (size_t) ret >= buflen - len)
+		return len;
+	len += ret;
 	switch (data->phase2_type) {
 	case EAP_TTLS_PHASE2_EAP:
-		len += snprintf(buf + len, buflen - len, "EAP-%s\n",
-				data->phase2_method ? data->phase2_method->name
-				: "?");
+		ret = snprintf(buf + len, buflen - len, "EAP-%s\n",
+			       data->phase2_method ? data->phase2_method->name
+			       : "?");
 		break;
 	case EAP_TTLS_PHASE2_MSCHAPV2:
-		len += snprintf(buf + len, buflen - len, "MSCHAPV2\n");
+		ret = snprintf(buf + len, buflen - len, "MSCHAPV2\n");
 		break;
 	case EAP_TTLS_PHASE2_MSCHAP:
-		len += snprintf(buf + len, buflen - len, "MSCHAP\n");
+		ret = snprintf(buf + len, buflen - len, "MSCHAP\n");
 		break;
 	case EAP_TTLS_PHASE2_PAP:
-		len += snprintf(buf + len, buflen - len, "PAP\n");
+		ret = snprintf(buf + len, buflen - len, "PAP\n");
 		break;
 	case EAP_TTLS_PHASE2_CHAP:
-		len += snprintf(buf + len, buflen - len, "CHAP\n");
+		ret = snprintf(buf + len, buflen - len, "CHAP\n");
+		break;
+	default:
+		ret = 0;
 		break;
 	}
+	if (ret < 0 || (size_t) ret >= buflen - len)
+		return len;
+	len += ret;
 
 	return len;
 }
