@@ -997,7 +997,7 @@ void eapol_sm_configure(struct eapol_sm *sm, int heldPeriod, int authPeriod,
 int eapol_sm_get_status(struct eapol_sm *sm, char *buf, size_t buflen,
 			int verbose)
 {
-	int len;
+	int len, ret;
 	if (sm == NULL)
 		return 0;
 
@@ -1006,21 +1006,26 @@ int eapol_sm_get_status(struct eapol_sm *sm, char *buf, size_t buflen,
 		       "suppPortStatus=%s\n",
 		       eapol_supp_pae_state(sm->SUPP_PAE_state),
 		       eapol_port_status(sm->suppPortStatus));
+	if (len < 0 || (size_t) len >= buflen)
+		return 0;
 
 	if (verbose) {
-		len += snprintf(buf + len, buflen - len,
-				"heldPeriod=%u\n"
-				"authPeriod=%u\n"
-				"startPeriod=%u\n"
-				"maxStart=%u\n"
-				"portControl=%s\n"
-				"Supplicant Backend state=%s\n",
-				sm->heldPeriod,
-				sm->authPeriod,
-				sm->startPeriod,
-				sm->maxStart,
-				eapol_port_control(sm->portControl),
-				eapol_supp_be_state(sm->SUPP_BE_state));
+		ret = snprintf(buf + len, buflen - len,
+			       "heldPeriod=%u\n"
+			       "authPeriod=%u\n"
+			       "startPeriod=%u\n"
+			       "maxStart=%u\n"
+			       "portControl=%s\n"
+			       "Supplicant Backend state=%s\n",
+			       sm->heldPeriod,
+			       sm->authPeriod,
+			       sm->startPeriod,
+			       sm->maxStart,
+			       eapol_port_control(sm->portControl),
+			       eapol_supp_be_state(sm->SUPP_BE_state));
+		if (ret < 0 || (size_t) ret >= buflen - len)
+			return len;
+		len += ret;
 	}
 
 	len += eap_sm_get_status(sm->eap, buf + len, buflen - len, verbose);
@@ -1044,10 +1049,11 @@ int eapol_sm_get_status(struct eapol_sm *sm, char *buf, size_t buflen,
 int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 {
 	size_t len;
+	int ret;
 
 	if (sm == NULL)
 		return 0;
-	len = snprintf(buf, buflen,
+	ret = snprintf(buf, buflen,
 		       "dot1xSuppPaeState=%d\n"
 		       "dot1xSuppHeldPeriod=%u\n"
 		       "dot1xSuppAuthPeriod=%u\n"
@@ -1064,10 +1070,11 @@ int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 		       "Authorized" : "Unauthorized",
 		       sm->SUPP_BE_state);
 
-	if (len >= buflen)
-		return len;
+	if (ret < 0 || (size_t) ret >= buflen)
+		return 0;
+	len = ret;
 
-	len += snprintf(buf + len, buflen - len,
+	ret = snprintf(buf + len, buflen - len,
 		       "dot1xSuppEapolFramesRx=%u\n"
 		       "dot1xSuppEapolFramesTx=%u\n"
 		       "dot1xSuppEapolStartFramesTx=%u\n"
@@ -1091,6 +1098,10 @@ int eapol_sm_get_mib(struct eapol_sm *sm, char *buf, size_t buflen)
 		       sm->dot1xSuppLastEapolFrameVersion,
 		       MAC2STR(sm->dot1xSuppLastEapolFrameSource));
 
+	if (ret < 0 || (size_t) ret >= buflen - len)
+		return len;
+	len += ret;
+
 	return len;
 }
 #endif /* CONFIG_CTRL_IFACE */
@@ -1663,6 +1674,21 @@ eapol_sm_get_config_blob(void *ctx, const char *name)
 }
 
 
+static void eapol_sm_notify_pending(void *ctx)
+{
+	struct eapol_sm *sm = ctx;
+	if (sm == NULL)
+		return;
+	if (sm->eapReqData && !sm->eapReq) {
+		wpa_printf(MSG_DEBUG, "EAPOL: received notification from EAP "
+			   "state machine - retrying pending EAP Request");
+		sm->eapolEap = TRUE;
+		sm->eapReq = TRUE;
+		eapol_sm_step(sm);
+	}
+}
+
+
 static struct eapol_callbacks eapol_cb =
 {
 	eapol_sm_get_config,
@@ -1672,7 +1698,8 @@ static struct eapol_callbacks eapol_cb =
 	eapol_sm_set_int,
 	eapol_sm_get_eapReqData,
 	eapol_sm_set_config_blob,
-	eapol_sm_get_config_blob
+	eapol_sm_get_config_blob,
+	eapol_sm_notify_pending
 };
 
 
