@@ -27,6 +27,7 @@
 #include "pmksa_cache.h"
 #include "wpa_ctrl.h"
 #include "eap.h"
+#include "ctrl_iface_dbus.h"
 
 
 static int wpa_supplicant_select_config(struct wpa_supplicant *wpa_s)
@@ -412,7 +413,10 @@ wpa_supplicant_select_bss(struct wpa_supplicant *wpa_s, struct wpa_ssid *group,
 			     memcmp(bss->bssid, ssid->bssid, ETH_ALEN) == 0) &&
 			    ((ssid->key_mgmt & WPA_KEY_MGMT_NONE) ||
 			     (ssid->key_mgmt & WPA_KEY_MGMT_IEEE8021X_NO_WPA))
-			    && bss->wpa_ie_len == 0 && bss->rsn_ie_len == 0 &&
+			    &&
+			    (!(ssid->key_mgmt &
+			       (WPA_KEY_MGMT_IEEE8021X | WPA_KEY_MGMT_PSK)) ||
+			     (bss->wpa_ie_len == 0 && bss->rsn_ie_len == 0)) &&
 			    wpa_supplicant_match_privacy(bss, ssid) &&
 			    !(bss->caps & IEEE80211_CAP_IBSS))
 			{
@@ -447,6 +451,9 @@ static void wpa_supplicant_event_scan_results(struct wpa_supplicant *wpa_s)
 		timeout = 1;
 		goto req_scan;
 	}
+
+	wpa_supplicant_dbus_notify_scan_results(wpa_s);
+
 	if (wpa_s->conf->ap_scan == 2)
 		return;
 	results = wpa_s->scan_results;
@@ -655,6 +662,7 @@ static void wpa_supplicant_event_assoc(struct wpa_supplicant *wpa_s,
 		/* Timeout for receiving the first EAPOL packet */
 		wpa_supplicant_req_auth_timeout(wpa_s, 10, 0);
 	}
+	wpa_supplicant_cancel_scan(wpa_s);
 }
 
 
@@ -685,13 +693,13 @@ static void wpa_supplicant_event_disassoc(struct wpa_supplicant *wpa_s)
 		bssid = wpa_s->pending_bssid;
 	wpa_blacklist_add(wpa_s, bssid);
 	wpa_sm_notify_disassoc(wpa_s->wpa);
-	wpa_supplicant_mark_disassoc(wpa_s);
 	wpa_msg(wpa_s, MSG_INFO, WPA_EVENT_DISCONNECTED "- Disconnect event - "
 		"remove keys");
 	if (wpa_supplicant_dynamic_keys(wpa_s)) {
 		wpa_s->keys_cleared = 0;
 		wpa_clear_keys(wpa_s, wpa_s->bssid);
 	}
+	wpa_supplicant_mark_disassoc(wpa_s);
 }
 
 
