@@ -326,7 +326,11 @@ struct scard_data * scard_init(scard_sim_type sim_type)
 	long ret;
 	unsigned long len;
 	struct scard_data *scard;
+#ifdef CONFIG_NATIVE_WINDOWS
+	TCHAR *readers = NULL;
+#else /* CONFIG_NATIVE_WINDOWS */
 	char *readers = NULL;
+#endif /* CONFIG_NATIVE_WINDOWS */
 	unsigned char buf[100];
 	size_t blen;
 	int transaction = 0;
