@@ -582,7 +582,7 @@ static int tlsv1_process_diffie_hellman(struct tlsv1_client *conn,
 		goto fail;
 	conn->dh_p_len = WPA_GET_BE16(pos);
 	pos += 2;
-	if (conn->dh_p_len == 0 || end - pos < conn->dh_p_len)
+	if (conn->dh_p_len == 0 || end - pos < (int) conn->dh_p_len)
 		goto fail;
 	conn->dh_p = malloc(conn->dh_p_len);
 	if (conn->dh_p == NULL)
@@ -596,7 +596,7 @@ static int tlsv1_process_diffie_hellman(struct tlsv1_client *conn,
 		goto fail;
 	conn->dh_g_len = WPA_GET_BE16(pos);
 	pos += 2;
-	if (conn->dh_g_len == 0 || end - pos < conn->dh_g_len)
+	if (conn->dh_g_len == 0 || end - pos < (int) conn->dh_g_len)
 		goto fail;
 	conn->dh_g = malloc(conn->dh_g_len);
 	if (conn->dh_g == NULL)
@@ -612,7 +612,7 @@ static int tlsv1_process_diffie_hellman(struct tlsv1_client *conn,
 		goto fail;
 	conn->dh_ys_len = WPA_GET_BE16(pos);
 	pos += 2;
-	if (conn->dh_ys_len == 0 || end - pos < conn->dh_ys_len)
+	if (conn->dh_ys_len == 0 || end - pos < (int) conn->dh_ys_len)
 		goto fail;
 	conn->dh_ys = malloc(conn->dh_ys_len);
 	if (conn->dh_ys == NULL)
@@ -1985,20 +1985,23 @@ int tlsv1_client_get_keyblock_size(struct tlsv1_client *conn)
 }
 
 
-int tlsv1_client_set_anon_dh(struct tlsv1_client *conn)
+int tlsv1_client_set_cipher_list(struct tlsv1_client *conn, u8 *ciphers)
 {
 #ifdef EAP_FAST
 	size_t count;
 	u16 *suites;
 
-	count = 0;
-	suites = conn->cipher_suites;
-	suites[count++] = TLS_DH_anon_WITH_AES_256_CBC_SHA;
-	suites[count++] = TLS_DH_anon_WITH_AES_128_CBC_SHA;
-	suites[count++] = TLS_DH_anon_WITH_3DES_EDE_CBC_SHA;
-	suites[count++] = TLS_DH_anon_WITH_RC4_128_MD5;
-	suites[count++] = TLS_DH_anon_WITH_DES_CBC_SHA;
-	conn->num_cipher_suites = count;
+	/* TODO: implement proper configuration of cipher suites */
+	if (ciphers[0] == TLS_CIPHER_ANON_DH_AES128_SHA) {
+		count = 0;
+		suites = conn->cipher_suites;
+		suites[count++] = TLS_DH_anon_WITH_AES_256_CBC_SHA;
+		suites[count++] = TLS_DH_anon_WITH_AES_128_CBC_SHA;
+		suites[count++] = TLS_DH_anon_WITH_3DES_EDE_CBC_SHA;
+		suites[count++] = TLS_DH_anon_WITH_RC4_128_MD5;
+		suites[count++] = TLS_DH_anon_WITH_DES_CBC_SHA;
+		conn->num_cipher_suites = count;
+	}
 
 	return 0;
 #else /* EAP_FAST */
